<?php
/**
 * @copyright  Copyright 2018 © Intelligent IT SRL. All rights reserved.
 */

class SmartBillUtils {
    var $stocks=null;
    var $order_currency;
    /**
     * Function is used to validate connection to SmartBill Cloud by querying the VAT status
     *
     * @return $input
     */
    public function validate_connection($options) {
        $status = false;

        try {
            if ( empty($options['SMARTBILL_CIF'])) throw new Exception(Context::getContext()->getTranslator()->trans('Va rugam sa completati toate datele din sectiunea de autentificare.', array(), 'Modules.Smartbill.Admin'));
            $client = new SmartBill_Cloud_REST_Client($options['SMARTBILL_EMAIL'], $options['SMARTBILL_API_TOKEN']);
            $taxes = $client->getTaxes($options['SMARTBILL_CIF']);
            Configuration::updateValue('SMARTBILL_TAXES',serialize($taxes));
            if (is_array($taxes) && isset($taxes['taxes'])){
                $status = true;
            } else {
                $status = Context::getContext()->getTranslator()->trans('Eroare la conectare la SmartBill Cloud pentru afisarea setarilor de TVA', array(), 'Modules.Smartbill.Admin');
            }
            Configuration::deleteByName('SMARTBILL_NO_TAX');
        } catch (Exception $e) {
            if ($e->getMessage() == 'Firma este neplatitoare de tva.') {
                Configuration::updateValue('SMARTBILL_NO_TAX',$e->getMessage());
                $status = true;
            } else {
                $status = $e->getMessage();
                Configuration::deleteByName('SMARTBILL_NO_TAX');
            }
            Configuration::deleteByName('SMARTBILL_TAXES');
        }

        return $status;
    }


    public function generateInvoiceData($order_id){
        $order = new Order($order_id);
        $options=SmartBillSettings::get_config_fields_values();
        $client_data = json_decode(json_encode($this->_getOrderClientData($order, $options)), true);
        
        $document_series = $options['SMARTBILL_INVOICE_SERIES'];
        if ((int)$options['SMARTBILL_DOCUMENT_TYPE'] === SMRT_DOCUMENT_TYPE_ESTIMATE) {
            $document_series = $options['SMARTBILL_ESTIMATE_SERIES'];
        }
        $is_draft = $options['SMARTBILL_INVOICE_IS_DRAFT'];
        $due_days = (int)$options['SMARTBILL_DUE_DAYS'];
        $delivery_days = (int)$options['SMARTBILL_DELIVERY_DAYS'];
        $currency = $options['SMARTBILL_BILLING_CURRENCY'];
        $smartbill_product = (bool)$options['SMARTBILL_PRODUCT'];
        $um = $options['SMARTBILL_UM'];
        $language=$options['SMARTBILL_INVOICE_LANG']==false?'RO':$options['SMARTBILL_INVOICE_LANG'];
        $use_intra_cif=$options['USE_INTRA_CIF']=='intracom'?true:false;
        $products_obj = $this->_getOrderProducts($order, $options);
        $products = [];
        foreach ($products_obj as $product) {
            $array_product = json_decode(json_encode($product), true);
            if($smartbill_product &&  !$product->isDiscount){
                $array_product['useSBProductName']=true;
            }
            $products[] = $array_product;
        }
        if (empty($due_days)) {
            $due_days = 0;
        }
        if (empty($delivery_days)) {
            $delivery_days = 0;
        }

        $smartbillInvoice = [
            'companyVatCode'    => $options['SMARTBILL_CIF'],
            'client' 		    => $client_data,
            'currency'          => $currency,
            'issueDate'         => date('Y-m-d'),
            'isDraft' 		    => $is_draft,
            'language'          => $language,
            'products' 		    => $products,
            'seriesName' 	    => $document_series,
            'useStock'          => false
        ];

        if("1" == $options['SMARTBILL_DOCUMENT_DATE']){
            $smartbillInvoice['issueDate'] = date('Y-m-d',strtotime($order->date_add));
        }

        if($use_intra_cif==true && $client_data['country'] != 'Romania'){
            $smartbillInvoice['useIntraCif'] = true;
        }

        if($options['SMARTBILL_SEND_MAIL_WITH_DOCUMENT'] == '1'){
            $smartbillInvoice['sendEmail']= true;
            $cc_bcc=array();
            $cc_bcc['cc']=$options['SMARTBILL_SEND_MAIL_CC'];
            $cc_bcc['bcc']=$options['SMARTBILL_SEND_MAIL_BCC'];
            $smartbillInvoice['email'] = $cc_bcc;
        }

        if ($options['SMARTBILL_ISSUE_WITH_DELIVERY_DAYS']){
            $smartbillInvoice['deliveryDate'] = date('Y-m-d', time() + abs((int)$delivery_days) * 24 * 3600);
        }
        if ($options['SMARTBILL_ISSUE_WITH_DUE_DATE']){
            $smartbillInvoice['dueDate'] = date('Y-m-d', time() + abs((int)$due_days) * 24 * 3600);
        }

        if ($options['SMARTBILL_USE_PAYMENT_TAX']) {
            $smartbillInvoice['usePaymentTax'] = true;
            $smartbillInvoice['paymentBase'] = 0;
            $smartbillInvoice['colectedTax'] = 0;
            $smartbillInvoice['paymentTotal'] = 0;
        } else {
            $smartbillInvoice['usePaymentTax'] = false;
        }

        if ( ! empty($options['SMARTBILL_STOCK']) ) {
            $smartbillInvoice['useStock'] = true;
        }

        if("1" == $options['SMARTBILL_INVOICE_CASHING'] && "1" == $order->hasBeenPaid()){
            $smartbillInvoice['payment'] = array(
                'value'  => $order->getTotalPaid(),
                'type'   => 'Card online',
                'isCash' => false,
            );
        }

        if("1" == $options['SMARTBILL_PAYMENT_URL']){
            $smartbillInvoice['paymentUrl'] = "Generate URL";
        }

        
        if('1' == $options['SMARTBILL_ADD_DELEGATE_DATA']){
            $options['SMARTBILL_ISSUER_NAME'] = trim($options['SMARTBILL_ISSUER_NAME']);
            if ( !empty($options['SMARTBILL_ISSUER_NAME']) ) {
                $smartbillInvoice['issuerName'] = $options['SMARTBILL_ISSUER_NAME'];
            }
            $options['SMARTBILL_ISSUER_CNP'] = trim($options['SMARTBILL_ISSUER_CNP']);
            if ( !empty($options['SMARTBILL_ISSUER_CNP']) ) {
                $smartbillInvoice['issuerCnp'] = $options['SMARTBILL_ISSUER_CNP'];
            }
            $options['SMARTBILL_DELEGATE_NAME'] = trim($options['SMARTBILL_DELEGATE_NAME']);
            if ( !empty($options['SMARTBILL_DELEGATE_NAME']) ) {
                $smartbillInvoice['delegateName'] = $options['SMARTBILL_DELEGATE_NAME'];
            }
            $options['SMARTBILL_DELEGATE_BULLETIN'] = trim($options['SMARTBILL_DELEGATE_BULLETIN']);
            if ( !empty($options['SMARTBILL_DELEGATE_BULLETIN']) ) {
                $smartbillInvoice['delegateIdentityCard'] = $options['SMARTBILL_DELEGATE_BULLETIN'];
            }
            $options['SMARTBILL_DELEGATE_AUTO'] = trim($options['SMARTBILL_DELEGATE_AUTO']);
            if ( !empty($options['SMARTBILL_DELEGATE_AUTO']) ) {
                $smartbillInvoice['delegateAuto'] = $options['SMARTBILL_DELEGATE_AUTO'];
            }
        }

        if(!empty($options['SMARTBILL_SHOW_ORDER_MENTION'])){
			$smartbillInvoice['mentions'] = str_replace( '#nr_comanda_online#', $order_id, $options['SMARTBILL_SHOW_ORDER_MENTION'] );
            $smartbillInvoice['mentions'] = str_replace( '#tip_plata#', $order->payment , $smartbillInvoice['mentions'] );
        }
        if(!empty($options['SMARTBILL_SHOW_ORDER_OBS'])){
            $smartbillInvoice['observations'] = str_replace( '#nr_comanda_online#', $order_id, $options['SMARTBILL_SHOW_ORDER_OBS'] );
        }
        
        if ( (int)$options['SMARTBILL_DOCUMENT_TYPE'] === SMRT_DOCUMENT_TYPE_ESTIMATE ) {
            unset($smartbillInvoice['useStock']);
        }

        return $smartbillInvoice;
    }

    // Get client data
    public function _getOrderClientData($order, $options) {
        $addressData        = new Address((int)$order->id_address_invoice);
        $addressState       = !empty($addressData->id_state) ? new State((int)$addressData->id_state) : false;
        $customerData       = $order->getCustomer();

        $client = new stdClass;
        $client->vatCode    = self::_getVatCode($addressData);
        if (empty($client->vatCode)) {
            $client->vatCode = trim($addressData->dni);
        }
        $client->name       = trim($addressData->company);
        $clientClean        = preg_replace('/[^a-z0-9]+/i', '',$client->name);
        $client->name       = empty($client->name) || empty($clientClean) ? $addressData->lastname.' '.$addressData->firstname : $client->name;
        $client->code       = '';
        $client->address    = $addressData->address1.', '.$addressData->address2.', '.$addressData->postcode;
        $client->regCom     = '';
        $client->isTaxPayer = strtolower(substr($client->vatCode,0,2))=='ro'?true:false;
        $client->contact    = $addressData->lastname.' '.$addressData->firstname;
        $client->phone      = !empty($addressData->phone) ? $addressData->phone : '';
        $client->city       = $addressData->city;
        $client->county     = !empty($addressState) ? $addressState->name : '';
        $client->country    = $addressData->country;
        $client->email      = !empty($customerData->email) ? $customerData->email : '';
        $client->bank       = '';
        $client->iban       = '';
        $client->saveToDb   = $options['SMARTBILL_SAVE_CLIENT'];

        return $client;
    }
    private static function _getVatCode($addressData) {
        $vatCode = trim($addressData->vat_number);
        $vatCodeClean = preg_replace('/[^0-9]+/i', '', $vatCode);

        if (empty($vatCodeClean)) {
            $vatCode = '';
        }

        return $vatCode;
    }

    // Get all products from order
    private function _getOrderProducts($order, $options) {
        $products = [];
        $include_discounts = $options['SMARTBILL_PRICE_INCLUDE_DISCOUNTS'];
        $orderProducts = $order->getProducts();

        foreach (array_reverse($order->getOrderDetailList()) as $product) {
            $products[] = $this->createOrderProduct($orderProducts[$product['id_order_detail']], $options);

            if ($include_discounts ) {
                $product_discount = $this->createOrderProductDiscount($orderProducts[$product['id_order_detail']], $product, $options);
                if ( !empty($product_discount) ) {
                    $products[] = $product_discount;
                }
            }
        }

        $this->order_currency = Currency::getCurrency($order->id_currency);

        // order discounts
        $cart_rule_index=0;
        foreach ( $order->getCartRules() as $cartRule ) {
            $temp_prod=$this->createOrderCartRule($cartRule, $options);
            $temp_prod->numberOfItems=count($products);
            $products[] = $temp_prod;
            $cart_rule_index++;
        }

        // ecotax
        $ecotax = $this->createOrderEcoTax($order, $options);
        if ( !empty($ecotax) ) {
            $products[] = $ecotax;
        }

        // wrapping
        $wrapping = $this->createOrderWrapping($order, $options);
        if ( !empty($wrapping) ) {
            $products[] = $wrapping;
        }

        // shipping
        if ($options['SMARTBILL_INCLUDE_SHIPPING']) {
            $transport = $this->createOrderTransport($options['SMARTBILL_SHIPPING_NAME'], $options['SMARTBILL_SHIPPING_NAME'], 1, $order, $options);

            if ( !empty($transport) ) {
                $products[] = $transport;
            }
        }

        return $products;
    }

    // Get product id to be sent to SMARTBILL (reference, EAN, SKU or product id)
    private function _getProductCode($order_item, $options) {
        $code = $options['SMARTBILL_PRODUCT_SKU_TYPE'];
        if ( !empty($code) ) {
            $code = $this->_productCode($order_item, $code);

            if ( empty($code) ) {
                $codes = array(SmartBillSettings::SKU_REFERENCE, SmartBillSettings::SKU_EAN13_JAN, SmartBillSettings::SKU_UPC);

                foreach ($codes as $key) {
                    $code = $this->_productCode($order_item, $key);
                    if ( !empty($code) ) break;
                }
            }
        }

        if ( empty($code) ) {
            $code = $order_item['product_id'];
        }

        return $code;
    }
    private function _productCode($order_item, $code) {
        $key  = 'product_' . $code;

        return !empty($order_item[$key]) ? $order_item[$key] : $order_item[$code];
    }

    // Generate Product object from order item
    private function createOrderProduct($order_item, $options) {
        if (!$options['SMARTBILL_PRICE_INCLUDE_DISCOUNTS']) {
            $product_price = (float)$order_item['unit_price_tax_incl'];
        } else {
            if (! empty ((float)$order_item['reduction_amount_tax_incl'])) {
                $product_price = (float)$order_item['unit_price_tax_incl'] + (float)$order_item['reduction_amount_tax_incl'];
            }
            if (! empty ((float)$order_item['reduction_percent'])) {
                $product_price = (float)$order_item['unit_price_tax_incl'] / (1 - (float)$order_item['reduction_percent']/100);
            }
        }
        if (!$product_price) {
            $product_price = (float)$order_item['unit_price_tax_incl'];
        }

        // ecotax is added separately
        if ( ! empty((float)$order_item['ecotax']) ) {
            if ( ! empty((float)$order_item['ecotax_tax_rate'])) {
                $product_price = $product_price - ((float)$order_item['ecotax']*(1+(float)$order_item['ecotax_tax_rate']/100));
            } else {
                $product_price = $product_price - (float)$order_item['ecotax'];
            }
        }

        $product = new stdClass;
        $product->code                      = $this->_getProductCode($order_item, $options);
        $product->currency                  = $options['SMARTBILL_BILLING_CURRENCY'];
        $product->exchangeRate              = 1;
        $product->isDiscount                = false;
        $product->isTaxIncluded             = true;
        $product->name                      = $order_item['product_name'];

        if('virtual' == $order_item['product_type']){
            $product->isService             = true;
        }

        if ( "no_value" == strtolower($options['SMARTBILL_UM']) ){
            throw new \Exception("Verifica setarile modulului SmartBill.");
        }

        $product->measuringUnitName = "@@@@@";
        if("preluata-din-smartbill" != strtolower($options['SMARTBILL_UM']) ){
            $product->measuringUnitName = $options['SMARTBILL_UM'];
        }

        if (! isset($product->measuringUnitName)){
            if ($options['SMARTBILL_STOCK']==''){
                $options['SMARTBILL_STOCK']='"Nu"';
            }
            throw new \Exception("Produsul {$product->name} nu are unitate de masura in gestiunea {$options['SMARTBILL_STOCK']}");
        }
       
        $product->price                     = $product_price;
        $product->quantity                  = (int)$order_item['product_quantity'];
        $product->saveToDb                  = $options['SMARTBILL_SAVE_PRODUCT'];
        $product->taxName                   = '';
        $product->taxPercentage             = $options['SMARTBILL_PRICES_VAT'];
        // VAT per product
        $vatParse=str_replace('-','',$options['SMARTBILL_PRICES_VAT']);
        $vatParse=trim($vatParse);
        if ($vatParse=='Preluata din Prestashop') {
            $product->taxPercentage = $order_item['tax_rate'];
            $tax = $this->findTaxName($order_item['tax_name'], $product->taxPercentage);
            if(2 == count($tax)){
                $product->taxName         = $tax['name'];
                $product->taxPercentage   = $tax['percentage'];
            }
        }

        $product->translatedName            = '';
        $product->translatedMeasuringUnit   = '';
        $product->warehouseName             = $options['SMARTBILL_STOCK'];

        // $product->price = number_format($product->price, 2, '.', '');
        return $product;
    }

    private function findTaxName( $tax_name, $tax_value){
        $taxes = unserialize(Configuration::get('SMARTBILL_VAT_RATES'));
        $return = [];
        
        foreach($taxes as $tax){
            if($tax['percentage'] == $tax_value && !isset($return['name'])){
                $return['percentage'] = $tax['percentage'];
                $return['name'] = $tax['name'];
            }

            if($tax['name'] == $tax_name && isset($return['percentage'])){
                $return['name'] = $tax['name'];
                break;
            }
        }
       
        return $return;
    }

    // Cart discounts
    private function createOrderCartRule($cartRule, $options) {
        $product = new stdClass;
        $product->code                      = Context::getContext()->getTranslator()->trans('coupon', array(), 'Modules.Smartbill.Admin');;
        $product->currency                  = $options['SMARTBILL_BILLING_CURRENCY'];
        $product->exchangeRate              = 1;
        $product->isDiscount                = true;
        $product->discountPercentage        = 0;
        $product->discountValue             = - abs((float)$cartRule['value']);
        $product->discountType              = !empty($product->discountValue) ? 1 : 2;
        $product->isTaxIncluded             = true;
        $product->measuringUnitName         = 'buc';
        $umVals=['no_value', "preluata-din-smartbill"];
        if(!in_array(strtolower($options['SMARTBILL_UM']),$umVals,true)){
            $product->measuringUnitName = $options['SMARTBILL_UM'];
        }
        $product->name                      = str_replace('#nume_cupon#',$cartRule['name'],$options['SMARTBILL_COUPON_TEXT']); //$cartRule['name'];
        $product->price                     = 0;
        $product->quantity                  = 1;
        $product->numberOfItems             = 1;
        $product->saveToDb                  = false;
        $product->taxName                   = '';
        $product->taxPercentage             = $options['SMARTBILL_PRICES_VAT'];
        $product->translatedMeasuringUnit   = '';
        $product->translatedName            = '';
        $product->warehouseName             = $options['SMARTBILL_STOCK'];

        // $product->discountValue = number_format($product->discountValue, 2, '.', '');

        return $product;
    }

    // Create ecotax (timbrul verde)
    private function createOrderEcoTax($order, $options) {
        $details = array(
            'ecotax'    => 0,
            'tax_rate'  => 0,
        );

        foreach ( $order->getProducts() as $product ) {
            if ( empty($product['ecotax']) )    continue;

            $details['ecotax']  += $product['product_quantity'] * $product['ecotax'];
            $details['tax_rate'] = (float)$product['ecotax_tax_rate'];
        }
        $details['ecotax'] = (float)$details['ecotax'];

        if ( empty($details['ecotax']) ) {
            return false;
        }

        $product = new stdClass;
        $product->code                      = 'ecotax';
        $product->currency                  = $options['SMARTBILL_BILLING_CURRENCY'];
        $product->exchangeRate              = 1;
        $product->isDiscount                = false;
        $product->isTaxIncluded             = true;
        $product->measuringUnitName         = 'buc';
        $product->name                      = 'Timbru Verde';
        $product->price                     = !empty($details['tax_rate']) ? $details['ecotax']*(1+$details['tax_rate']/100) : $details['ecotax'];
        $product->quantity                  = 1;
        $product->saveToDb                  = false;
        $product->taxName                   = '';
        $product->taxPercentage             = $details['tax_rate'];
        $product->translatedMeasuringUnit   = '';
        $product->translatedName            = '';
        $product->warehouseName             = $options['SMARTBILL_STOCK'];
        $product->isService                 = true;

        return $product;
    }

    // Create wrapping if is checked in the checkout process
    private function createOrderWrapping($order, $options) {
        $details = array(
            'wrapping'  => 0,
            'tax_rate'  => 0,
        );

        if ( !empty($order->total_wrapping)
          && 0 < $order->total_wrapping ) {
            $details['wrapping'] = $order->total_wrapping;

            $order->total_wrapping_tax_excl = (float)$order->total_wrapping_tax_excl;
            $order->total_wrapping_tax_incl = (float)$order->total_wrapping_tax_incl;
            if ( !empty($order->total_wrapping_tax_excl) ) {
                $details['tax_rate'] = ($order->total_wrapping_tax_incl - $order->total_wrapping_tax_excl) / $order->total_wrapping_tax_excl * 100;
                $details['tax_rate'] = round($details['tax_rate']);
            }
        }

        if ( empty($details['wrapping'])
          || 0 >= $details['wrapping'] ) {
              return false;
        }

        $product = new stdClass;
        $product->code                      = 'wrapping';
        $product->currency                  = $options['SMARTBILL_BILLING_CURRENCY'];
        $product->exchangeRate              = 1;
        $product->isDiscount                = false;
        $product->isTaxIncluded             = false;
        $product->measuringUnitName         = 'buc';
        $product->name                      = 'Impachetare';
        $product->price                     = (float)$details['wrapping'];
        $product->quantity                  = 1;
        $product->saveToDb                  = false;
        $product->taxName                   = '';
        $product->taxPercentage             = (float)$details['tax_rate'];
        $product->translatedMeasuringUnit   = '';
        $product->translatedName            = '';
        $product->warehouseName             = $options['SMARTBILL_STOCK'];
        $product->isService                 = true;

        return $product;
    }

    // Generate discount object from order item
    private function createOrderProductDiscount($order_item, $order_product, $options) {
        $product = null;

        $order_product['reduction_percent'] = isset($order_product['reduction_percent']) ? (float)$order_product['reduction_percent'] : 0;
        $order_product['reduction_amount'] = isset($order_product['reduction_amount']) ? (float)$order_product['reduction_amount'] : 0;
        
        if ( 0 < $order_product['reduction_percent'] || 0 < $order_product['reduction_amount'] ) {
            $product = new stdClass;
            $product->code                      = $this->_getProductCode($order_item, $options);
            $product->currency                  = $options['SMARTBILL_BILLING_CURRENCY'];
            $product->exchangeRate              = 1;
            $product->isTaxIncluded             = true;
            $product->isDiscount                = true;
            $product->discountPercentage        = 0 < $order_product['reduction_percent'] ? $order_product['reduction_percent'] : 0;
            $product->quantity                  = $order_item['product_quantity'];
            $product->discountValue             = 0;
            $product->discountType              = 2;
            if ( 0 < $order_product['reduction_amount'] ) {
                $product->discountValue         = $product->quantity * -abs($order_product['reduction_amount_tax_incl']);
                $product->discountType          = 1;
            }
            
            if ( "no_value" == strtolower($options['SMARTBILL_UM']) ){
                throw new \Exception("Verifica setarile modulului SmartBill.");
            }

            $product->measuringUnitName = "@@@@@";
            if("preluata-din-smartbill" != strtolower($options['SMARTBILL_UM']) ){
                $product->measuringUnitName = $options['SMARTBILL_UM'];
            }

            $smartbill_product = (bool)$options['SMARTBILL_PRODUCT'];
            $product->name     = 'Discount (pret special): '.$order_item['product_name'];
            if($smartbill_product){ $product->name = 'Discount'; }

            $product->price                     = 0;
            $product->numberOfItems             = 1;
            $product->saveToDb                  = false;
            $product->taxName                   = '';
            $product->taxPercentage             = $options['SMARTBILL_PRICES_VAT'];
            
            $vatParse=str_replace('-','',$options['SMARTBILL_PRICES_VAT']);
            $vatParse=trim($vatParse);

            if ($vatParse=='Preluata din Prestashop') {
                $product->taxPercentage = (float) $order_item['tax_rate'];
                $tax = $this->findTaxName( $order_item['tax_name'], $product->taxPercentage );
                if(2 == count($tax)){
                    $product->taxName         = $tax['name'];
                    $product->taxPercentage   = $tax['percentage'];
                }
            }


            $product->translatedMeasuringUnit   = '';
            $product->translatedName            = '';
            $product->warehouseName             = $options['SMARTBILL_STOCK'];

            // $product->discountValue = number_format($product->discountValue, 2, '.', '');
            // $product->discountTaxValue = number_format($product->discountTaxValue, 2, '.', '');
        }
       
        return $product;
    }

    // Generate shipping (transport) object
    private static function createOrderTransport($code, $name, $quantity, $order, $options)
    {
        $shipping = $order->getShipping();
        $shipping_price = $shipping[0]['shipping_cost_tax_incl'];
        
        if(1 == $options['SMARTBILL_FREE_SHIPPING'] && 0.0 == $shipping_price ){
            return null;
        }
        $product = new stdClass;
        $product->code                      = $code;
        $product->currency                  = $options['SMARTBILL_BILLING_CURRENCY'];
        $product->isDiscount                = false;
        $product->isTaxIncluded             = true;
        $product->measuringUnitName         = 'buc';
        $product->name                      = $name;
        $product->price                     = (float)$shipping_price;
        $product->quantity                  = $quantity;
        $product->saveToDb                  = false;

        $product->taxName                   = '';
        $product->taxPercentage             = $options['SMARTBILL_TRANSPORT_VAT'];
       
        if($options['SMARTBILL_TRANSPORT_VAT']=='presta' && $shipping[0]['shipping_cost_tax_excl'] >0){
            $product->taxPercentage = round(($shipping[0]['shipping_cost_tax_incl'] - $shipping[0]['shipping_cost_tax_excl'])/$shipping[0]['shipping_cost_tax_excl']*100,0);
        }
        $product->isService                 = true;

        // $product->price = number_format($product->price, 2, '.', '');

        return $product;
    }

    private static function export_order($order) {
        $order->products = $order->getOrderDetailList();
        $order = json_decode(json_encode($order), true);
        foreach ($order['products'] as $k => $product) {
            foreach ($product as $label => $value) {
                $order['line_item_'.$k.'_'.$label] = $value;
            }
        }
        unset($order['products']);
        unset($order['id_shop_list']);

        return $order;
    }
}
